/*
 *  Copyright (C) 2020 Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**
 *  \file sciserver_testapp_main.c
 *
 *  \brief Example Application for sciserver
 *
 */

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#include <ti/sysbios/BIOS.h>
#include <ti/board/board.h>
#include <ti/drv/sciclient/sciserver_tirtos.h>
#include <ti/drv/sciclient/examples/common/sciclient_appCommon.h>

/* ========================================================================== */
/*                           Macros & Typedefs                                */
/* ========================================================================== */

/** \brief Aligned address at which the X509 header is placed. */
#define SCISERVER_COMMON_X509_HEADER_ADDR (0x41cffb00)

/* ========================================================================== */
/*                         Structures and Enums                               */
/* ========================================================================== */

/* None */

/* ========================================================================== */
/*                 Internal Function Declarations                             */
/* ========================================================================== */

/* None */

/* ========================================================================== */
/*                            Global Variables                                */
/* ========================================================================== */

/* None */

/* ========================================================================== */
/*                          Function Definitions                              */
/* ========================================================================== */

#if defined(SOC_J7200)
static void setMpuFrequency(void)
{
    int32_t cslRet = CSL_EFAIL;
    int32_t tisci_dev_id = TISCI_DEV_A72SS0_CORE0_0;
    int32_t tisci_clk_id = TISCI_DEV_A72SS0_CORE0_0_ARM_CLK_CLK;
    int32_t slave_clk_freq_hz = 1200000000;
    uint64_t respFreq, currentFreq;
    uint32_t regRdVal;

    cslRet = Sciclient_pmQueryModuleClkFreq (tisci_dev_id,
                                             tisci_clk_id,
                                             slave_clk_freq_hz,
                                             &respFreq,
                                             SCICLIENT_SERVICE_WAIT_FOREVER);
    if (cslRet != CSL_PASS)
    {
        App_sciclientPrintf("Sciclient_pmQueryModuleClkFreq...FAILED \n");
    }
    else
    {
        App_sciclientPrintf("Sciclient_pmQueryModuleClkFreq, DevId 0x%x @ %dHz... \n", tisci_dev_id, respFreq);
    }

    /* PLL8_HSDIV0 output is selected and output to MAIN_PLL8_HSDIV0_CLKOUT */
    CSL_REG32_FINS(CSL_PLL0_CFG_BASE + CSL_MAIN_PLL_MMR_CFG_PLL8_SS_CTRL, MAIN_PLL_MMR_CFG_PLL8_SS_CTRL_WAVE_SEL, 0x1);
    App_sciclientPrintf("PLL8_SS_CTRL Register Value = 0x%x \n", CSL_REG32_RD(CSL_PLL0_CFG_BASE + CSL_MAIN_PLL_MMR_CFG_PLL8_SS_CTRL));

#if 0
    /* OBSCLK0_MUX1_CLKOUT = MAIN_PLL8_HSDIV0_CLKOUT / DIV8 */
    CSL_REG32_FINS(CSL_CTRL_MMR0_CFG0_BASE + CSL_MAIN_CTRL_MMR_CFG0_OBSCLK1_CTRL, MAIN_CTRL_MMR_CFG0_OBSCLK1_CTRL_CLK_SEL, 0x1);
    App_sciclientPrintf("CTRLMMR_OBSCLK1_CTRL Register Value = 0x%x \n", CSL_REG32_RD(CSL_CTRL_MMR0_CFG0_BASE + CSL_MAIN_CTRL_MMR_CFG0_OBSCLK1_CTRL));

    /* OBSCLK0_MUX0_CLKOUT = OBSCLK0_MUX1_CLKOUT. The clock is divided and output to the OBSCLK[2:0] pins */
    CSL_REG32_FINS(CSL_CTRL_MMR0_CFG0_BASE + CSL_MAIN_CTRL_MMR_CFG0_OBSCLK0_CTRL, MAIN_CTRL_MMR_CFG0_OBSCLK0_CTRL_CLK_SEL, 0xD);

    /* OBSCLK0 output clock is divided by 2 (clkdiv = 1). The maximum supported frequency is 200 MHz */
    CSL_REG32_FINS(CSL_CTRL_MMR0_CFG0_BASE + CSL_MAIN_CTRL_MMR_CFG0_OBSCLK0_CTRL, MAIN_CTRL_MMR_CFG0_OBSCLK0_CTRL_CLK_DIV, 0x1);

    /* Load the new divider value */
    CSL_REG32_FINS(CSL_CTRL_MMR0_CFG0_BASE + CSL_MAIN_CTRL_MMR_CFG0_OBSCLK0_CTRL, MAIN_CTRL_MMR_CFG0_OBSCLK0_CTRL_CLK_DIV_LD, 0x0);
    CSL_REG32_FINS(CSL_CTRL_MMR0_CFG0_BASE + CSL_MAIN_CTRL_MMR_CFG0_OBSCLK0_CTRL, MAIN_CTRL_MMR_CFG0_OBSCLK0_CTRL_CLK_DIV_LD, 0x1);

    App_sciclientPrintf("CTRLMMR_OBSCLK0_CTRL Register Value = 0x%x \n", CSL_REG32_RD(CSL_CTRL_MMR0_CFG0_BASE + CSL_MAIN_CTRL_MMR_CFG0_OBSCLK0_CTRL));
#endif

    App_sciclientPrintf("Sciclient_pmSetModuleClkFreq, DevId 0x%x @ %dHz... \n", tisci_dev_id, slave_clk_freq_hz);
    cslRet = Sciclient_pmSetModuleClkFreq(tisci_dev_id,
                                          tisci_clk_id,
                                          slave_clk_freq_hz,
                                          TISCI_MSG_FLAG_AOP,
                                          SCICLIENT_SERVICE_WAIT_FOREVER);
    if (cslRet != CSL_PASS)
    {
        App_sciclientPrintf("Sciclient_pmSetModuleClkFreq...FAILED \n");
    }

    cslRet = Sciclient_pmGetModuleClkFreq(tisci_dev_id,
                                          tisci_clk_id,
                                          &currentFreq,
                                          SCICLIENT_SERVICE_WAIT_FOREVER);
    if (cslRet != CSL_PASS)
    {
        App_sciclientPrintf("Sciclient_pmGetModuleClkFreq...FAILED \n");
    }
    else
    {
        App_sciclientPrintf("Sciclient_pmGetModuleClkFreq, DevId 0x%x @ %dHz... \n", tisci_dev_id, currentFreq);
    }

    regRdVal = CSL_REG32_RD(CSL_WKUP_CTRL_MMR0_CFG0_BASE + CSL_WKUP_CTRL_MMR_CFG0_MAIN_PLL8_CLKSEL);
    App_sciclientPrintf("CTRLMMR_WKUP_MAIN_PLL8_CLKSEL Register Value = 0x%x \n", regRdVal);
    App_sciclientPrintf("BYPASS_SW_OVRD Field Value = 0x%x \n", CSL_FEXT(regRdVal, WKUP_CTRL_MMR_CFG0_MAIN_PLL8_CLKSEL_BYPASS_SW_OVRD));
    App_sciclientPrintf("BYP_WARM_RST Field Value = 0x%x \n", CSL_FEXT(regRdVal, WKUP_CTRL_MMR_CFG0_MAIN_PLL8_CLKSEL_BYP_WARM_RST));
    App_sciclientPrintf("CLK_SEL Field Value = 0x%x \n", CSL_FEXT(regRdVal, WKUP_CTRL_MMR_CFG0_MAIN_PLL8_CLKSEL_CLK_SEL));

    regRdVal = CSL_REG32_RD(CSL_PLL0_CFG_BASE + CSL_MAIN_PLL_MMR_CFG_PLL8_FREQ_CTRL0);
    App_sciclientPrintf("PLL8_FREQ_CTRL0 Register Value = 0x%x \n", regRdVal);
    App_sciclientPrintf("FB_DIV_INT Field Value = 0x%x \n", CSL_FEXT(regRdVal, MAIN_PLL_MMR_CFG_PLL8_FREQ_CTRL0_FB_DIV_INT));

    regRdVal = CSL_REG32_RD(CSL_PLL0_CFG_BASE + CSL_MAIN_PLL_MMR_CFG_PLL8_FREQ_CTRL1);
    App_sciclientPrintf("PLL8_FREQ_CTRL1 Register Value = 0x%x \n", regRdVal);
    App_sciclientPrintf("FB_DIV_FRAC Field Value = 0x%x \n", CSL_FEXT(regRdVal, MAIN_PLL_MMR_CFG_PLL8_FREQ_CTRL1_FB_DIV_FRAC));

    regRdVal = CSL_REG32_RD(CSL_PLL0_CFG_BASE + CSL_MAIN_PLL_MMR_CFG_PLL8_DIV_CTRL);
    App_sciclientPrintf("PLL8_DIV_CTRL Register Value = 0x%x \n", regRdVal);
    App_sciclientPrintf("POST_DIV2 Field Value = 0x%x \n", CSL_FEXT(regRdVal, MAIN_PLL_MMR_CFG_PLL8_DIV_CTRL_POST_DIV2));
    App_sciclientPrintf("POST_DIV1 Field Value = 0x%x \n", CSL_FEXT(regRdVal, MAIN_PLL_MMR_CFG_PLL8_DIV_CTRL_POST_DIV1));
    App_sciclientPrintf("REF_DIV Field Value = 0x%x \n", CSL_FEXT(regRdVal, MAIN_PLL_MMR_CFG_PLL8_DIV_CTRL_REF_DIV));

    regRdVal = CSL_REG32_RD(CSL_PLL0_CFG_BASE + CSL_MAIN_PLL_MMR_CFG_PLL8_SS_CTRL);
    App_sciclientPrintf("PLL8_SS_CTRL Register Value = 0x%x \n", regRdVal);
    App_sciclientPrintf("BYPASS_EN Field Value = 0x%x \n", CSL_FEXT(regRdVal, MAIN_PLL_MMR_CFG_PLL8_SS_CTRL_BYPASS_EN));
    App_sciclientPrintf("WV_TBLE_MAXADDR Field Value = 0x%x \n", CSL_FEXT(regRdVal, MAIN_PLL_MMR_CFG_PLL8_SS_CTRL_WV_TBLE_MAXADDR));
    App_sciclientPrintf("RESET Field Value = 0x%x \n", CSL_FEXT(regRdVal, MAIN_PLL_MMR_CFG_PLL8_SS_CTRL_RESET));
    App_sciclientPrintf("DOWNSPREAD_EN Field Value = 0x%x \n", CSL_FEXT(regRdVal, MAIN_PLL_MMR_CFG_PLL8_SS_CTRL_DOWNSPREAD_EN));
    App_sciclientPrintf("WAVE_SEL Field Value = 0x%x \n", CSL_FEXT(regRdVal, MAIN_PLL_MMR_CFG_PLL8_SS_CTRL_WAVE_SEL));

    regRdVal = CSL_REG32_RD(CSL_PLL0_CFG_BASE + CSL_MAIN_PLL_MMR_CFG_PLL8_SS_SPREAD);
    App_sciclientPrintf("PLL8_SS_SPREAD Value = 0x%x \n", regRdVal);
    App_sciclientPrintf("MOD_DIV Field Value = 0x%x \n", CSL_FEXT(regRdVal, MAIN_PLL_MMR_CFG_PLL8_SS_SPREAD_MOD_DIV));
    App_sciclientPrintf("SPREAD Field Value = 0x%x \n", CSL_FEXT(regRdVal, MAIN_PLL_MMR_CFG_PLL8_SS_SPREAD_SPREAD));

    regRdVal = CSL_REG32_RD(CSL_PLL0_CFG_BASE + CSL_MAIN_PLL_MMR_CFG_PLL8_HSDIV_CTRL0);
    App_sciclientPrintf("PLL8_HSDIV_CTRL0 Register Value = 0x%x \n", regRdVal);
    App_sciclientPrintf("RESET Field Value = 0x%x \n", CSL_FEXT(regRdVal, MAIN_PLL_MMR_CFG_PLL8_HSDIV_CTRL0_RESET));
    App_sciclientPrintf("CLKOUT_EN Field Value = 0x%x \n", CSL_FEXT(regRdVal, MAIN_PLL_MMR_CFG_PLL8_HSDIV_CTRL0_CLKOUT_EN));
    App_sciclientPrintf("SYNC_DIS Field Value = 0x%x \n", CSL_FEXT(regRdVal, MAIN_PLL_MMR_CFG_PLL8_HSDIV_CTRL0_SYNC_DIS));
    App_sciclientPrintf("HSDIV Field Value = 0x%x \n", CSL_FEXT(regRdVal, MAIN_PLL_MMR_CFG_PLL8_HSDIV_CTRL0_HSDIV));
}
#endif

int main(void)
{
    int32_t ret = CSL_PASS;
    Sciclient_ConfigPrms_t clientPrms;
    Sciserver_TirtosCfgPrms_t appPrms;

    /* Sciclient needs to be initialized before Sciserver. Sciserver depends on
     * Sciclient API to execute message forwarding */
    ret = Sciclient_configPrmsInit(&clientPrms);

    if (ret == CSL_PASS)
    {
        ret = Sciclient_boardCfgParseHeader(
            (uint8_t *) SCISERVER_COMMON_X509_HEADER_ADDR,
            &clientPrms.inPmPrms, &clientPrms.inRmPrms);
    }
    
    if (ret == CSL_PASS)
    {
        ret = Sciclient_init(&clientPrms);
    }

    if (ret == CSL_PASS)
    {
        ret = Sciserver_tirtosInitPrms_Init(&appPrms);
    }

    if (ret == CSL_PASS)
    {
        ret = Sciserver_tirtosInit(&appPrms);
    }

    /* Enable UART console print*/
    if (ret == CSL_PASS)
    {
        App_sciclientConsoleInit();
    }

#if defined(SOC_J7200)
    setMpuFrequency();
#endif

    App_sciclientPrintf("Sciserver Built On: %s %s\n", __DATE__, __TIME__);
    if (ret == CSL_PASS)
    {
        App_sciclientPrintf("Starting Sciserver..... PASSED\n");

        uint32_t freqHz;
        Sciclient_pmGetModuleClkFreq(TISCI_DEV_GTC0, TISCI_DEV_GTC0_GTC_CLK,
            (uint64_t *) &freqHz, SCICLIENT_SERVICE_WAIT_FOREVER);
        App_sciclientPrintf("GTC freq: %d\n", freqHz);

        BIOS_start();
    }
    else
    {
        App_sciclientPrintf("Starting Sciserver..... FAILED\n");
    }
    return ret;
}

/* ========================================================================== */
/*                 Internal Function Definitions                              */
/* ========================================================================== */

/* None */

